/* target_generic_io.h - Native methods for generic I/O operations
   Copyright (C) 1998 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
 
GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

/*
Description: generic target defintions of miscellaneous functions
Systems    : all
*/

#ifndef __TARGET_GENERIC_IO__
#define __TARGET_GENERIC_IO__

/* check if target_native_io.h included */
#ifndef __TARGET_NATIVE_IO__
  #error Do NOT INCLUDE generic target files! Include the corresponding native target files instead!
#endif

/****************************** Includes *******************************/
/* do not move; needed here because of some macro definitions */
#include "config.h"

#include <stdlib.h>
#include <assert.h>

#include "target_native.h"

#ifdef NEW_CP
#include "../posix/target_posix_io.h"
#endif

/****************** Conditional compilation switches *******************/

/***************************** Constants *******************************/

/***************************** Datatypes *******************************/

/***************************** Variables *******************************/

/****************************** Macros *********************************/

/***********************************************************************\
* Name       : TARGET_NATIVE_IO_READ_STDIN
* Purpose    : read data from stdin
* Input      : -
* Output     : -
* Return     : -
* Side-effect: unknown
* Notes      : -
\***********************************************************************/

#ifndef NEW_CP
#ifndef TARGET_NATIVE_IO_READ_STDIN
  #ifndef HAVE_UNISTD_H
    #error no unistd.h: read() not available
  #endif
  #include <unistd.h>
  #define TARGET_NATIVE_IO_READ_STDIN(buffer,length,bytesRead,result) \
    do { \
      bytesRead=read(0,buffer,length); \
      result=(bytesRead!=-1)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
    } while (0)
#endif
#else /* NEW_CP */
    #define TARGET_NATIVE_IO_READ_STDIN(buffer,length,bytesWritten,result) \
      CP_IO_READ_STDIN(buffer,length,bytesWritten,result)
#endif /* NEW_CP */

/***********************************************************************\
* Name       : TARGET_NATIVE_IO_WRITE_STDOUT
* Purpose    : write data to stdout
* Input      : -
* Output     : -
* Return     : -
* Side-effect: unknown
* Notes      : -
\***********************************************************************/

#ifndef NEW_CP
#ifndef TARGET_NATIVE_IO_WRITE_STDOUT
  #ifndef HAVE_UNISTD_H
    #error no unistd.h: write() not available
  #endif
  #include <unistd.h>
  #define TARGET_NATIVE_IO_WRITE_STDOUT(buffer,length,bytesWritten,result) \
    do { \
      bytesWritten=write(1,buffer,length); \
      result=(bytesWritten!=-1)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
    } while (0)
#endif
#else /* NEW_CP */
    #define TARGET_NATIVE_IO_WRITE_STDOUT(buffer,length,bytesWritten,result) \
      CP_IO_WRITE_STDOUT(buffer,length,bytesWritten,result)
#endif /* NEW_CP */

/***********************************************************************\
* Name       : write data to stderr
* Purpose    : write data to stderr
* Input      : -
* Output     : -
* Return     : -
* Side-effect: unknown
* Notes      : -
\***********************************************************************/

#ifndef NEW_CP
#ifndef TARGET_NATIVE_IO_WRITE_STDERR
  #ifndef HAVE_UNISTD_H
    #error no unistd.h: write() not available
  #endif
  #include <unistd.h>
  #define TARGET_NATIVE_IO_WRITE_STDERR(buffer,length,bytesWritten,result) \
    do { \
      bytesWritten=write(2,buffer,length); \
      result=(bytesWritten!=-1)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
    } while (0)
#endif
#else /* NEW_CP */
    #define TARGET_NATIVE_IO_WRITE_STDERR(buffer,length,bytesWritten,result) \
      CP_IO_WRITE_STDERR(buffer,length,bytesWritten,result)
#endif /* NEW_CP */

/*---------------------------------------------------------------------*/

/***********************************************************************\
* Name       : TARGET_NATIVE_IO_PRINT
* Purpose    : print formated string to stdout
* Input      : format - format string (like printf)
*              args   - optional arguments
* Output     : -
* Return     : number of characters printed
* Side-effect: unknown
* Notes      : Even if printf() is an ANSI-C function a macro is used
*              to be able to use some other output channels than stdout
*              (e. g. a file or a serial port).
*
*              Hint: for non-gcc compilers only then name is defined and
*              a function is inserted, because of variable number of
*              arguments. Only the preprocessor of gcc support a
*              feature for defining macros with variable parameters. To
*              be able to use an other compiler the macro only replaces
*              the name with some function implementing the output of a
*              string with variable parameters (like printf)
\***********************************************************************/

/* __PUBLIC_BEGIN__ */
#ifndef NEW_CP
#ifndef TARGET_NATIVE_IO_PRINT
/* still not active, because of strange macro problem x=(TARGET_NATIVE_IO_PRINT_ERROR(...),b) */
  #ifdef x__GNUC__
    #include <stdio.h>
    #define TARGET_NATIVE_IO_PRINT(format,args...) \
      fprintf(stdout,format, ## args)
  #else
    #define TARGET_NATIVE_IO_PRINT_GENERIC
    #define TARGET_NATIVE_IO_PRINT targetGenericIO_printf
  #endif
#endif
#else /* NEW_CP */
    #define TARGET_NATIVE_IO_PRINT \
      CP_IO_PRINT
#endif /* NEW_CP */
/* __PUBLIC_END__ */

/***********************************************************************\
* Name       : TARGET_NATIVE_IO_PRINT_ERROR
* Purpose    : print formated string to stderr
* Input      : format - format string (like printf)
*              args   - optional arguments
* Output     : -
* Return     : number of characters printed
* Side-effect: unknown
* Notes      : Even if printf() is an ANSI-C function a macro is used
*              to be able to use some other output channels than stderr
*              (e. g. a file or a serial port).
*
*              Hint: for non-gcc compilers only then name is defined and
*              a function is inserted, because of variable number of
*              arguments. Only the preprocessor of gcc support a
*              feature for defining macros with variable parameters. To
*              be able to use an other compiler the macro only replaces
*              the name with some function implementing the output of a
*              string with variable parameters (like printf)
\***********************************************************************/

/* __PUBLIC_BEGIN__ */
#ifndef NEW_CP
#ifndef TARGET_NATIVE_IO_PRINT_ERROR
/* still not active, because of strange macro problem x=(TARGET_NATIVE_IO_PRINT_ERROR(...),b) */
  #ifdef x__GNUC__
    #include <stdio.h>
    #define TARGET_NATIVE_IO_PRINT_ERROR(format,args...) \
      fprintf(stderr,format, ## args)
  #else
    #define TARGET_NATIVE_IO_PRINT_ERROR_GENERIC
    #define TARGET_NATIVE_IO_PRINT_ERROR targetGenericIO_printf_stderr
  #endif
#endif
#else /* NEW_CP */
    #define TARGET_NATIVE_IO_PRINT_ERROR \
      CP_IO_PRINT_ERROR
#endif /* NEW_CP */
/* __PUBLIC_END__ */

/***************************** Functions *******************************/

#ifdef __cplusplus
extern "C" {
#endif

/* __PUBLIC_BEGIN__ */
#ifdef TARGET_NATIVE_IO_PRINT_GENERIC
int targetGenericIO_printf(const char *format, ...);
#endif /* TARGET_NATIVE_IO_PRINT_GENERIC */
/* __PUBLIC_END__ */

/* __PUBLIC_BEGIN__ */
#ifdef TARGET_NATIVE_IO_PRINT_ERROR_GENERIC
int targetGenericIO_printf_stderr(const char *format, ...);
#endif /* TARGET_NATIVE_IO_PRINT_ERROR_GENERIC */
/* __PUBLIC_END__ */

#ifdef __cplusplus
}
#endif

#endif /* __TARGET_GENERIC_IO__ */

/* end of file */
